from datetime import datetime
from typing import Dict, List, Union

from pydantic import BaseModel, Field


class UserDataStats(BaseModel):
    Dramas: str
    TV_Shows: str = Field(alias="TV Shows")
    Episodes: str
    Movies: str
    Days: str


class UserDataListItem(BaseModel):
    name: str
    id: str
    score: str
    episode_seen: str
    episode_total: str


class UserDataListGroup(BaseModel):
    items: List[UserDataListItem]
    stats: Union[UserDataStats, Dict[str, str]]


class UserDataList(BaseModel):
    Completed: UserDataListGroup
    Plan_to_Watch: UserDataListGroup = Field(alias="Plan to Watch")
    On_hold: UserDataListGroup = Field(alias="On-hold")
    Dropped: UserDataListGroup


class UserData(BaseModel):
    link: str
    list: Union[UserDataList, Dict[str, UserDataListGroup]]


class UserQuery(BaseModel):
    slug_query: str
    data: UserData
    scrape_date: datetime



from typing import List, TypeVar

from pydantic import BaseModel, TypeAdapter

BASE_KURYANA_API_URL = "https://kuryana.tbdh.app/"


T = TypeVar("T", bound=BaseModel)


class BaseClient:
    def _parse_response(self, data: str, class_type: type[T]) -> T:
        return class_type.model_validate_json(data, strict=False)

    def _parse_array_response(
        self, data: str, adapter_type: TypeAdapter[List[T]]
    ) -> List[T]:
        return adapter_type.validate_json(data, strict=False)



from typing import List, Optional

from pydantic import BaseModel, TypeAdapter


class SeasonalDrama(BaseModel):
    id: int
    title: str
    episodes: Optional[int] = None
    ranking: int
    popularity: int
    country: str
    content_type: str
    type: str
    synopsis: Optional[str] = None
    released_at: str
    url: str
    genres: str
    thumbnail: str
    cover: str
    rating: float
    timezone: str
    add_status: bool


SeasonalDramaQuery = TypeAdapter(List[SeasonalDrama])



from typing import Dict, List, Optional, Union

from pydantic import BaseModel, Field


class PeopleRole(BaseModel):
    name: str
    type: str


class PeopleTitle(BaseModel):
    link: str
    name: str


class PeopleDrama(BaseModel):
    _slug: str
    year: int
    title: PeopleTitle
    rating: float
    role: PeopleRole
    episodes: Optional[int] = None


class PeopleDetails(BaseModel):
    name: str
    native_name: str
    given_name: Optional[str] = None
    also_known_as: str
    nationality: str
    gender: str
    born: str
    age: str


class PeopleWorks(BaseModel):
    Drama: List[PeopleDrama]
    Movie: List[PeopleDrama]
    tv_show: List[PeopleDrama] = Field(alias="TV Show")


class PeopleData(BaseModel):
    link: str
    name: str
    about: str
    profile: str
    works: Union[PeopleWorks, Dict[str, List[PeopleDrama]]]
    details: PeopleDetails


class PeopleQuery(BaseModel):
    slug_query: str
    data: PeopleData
    scrape_date: str



import pytest

from kuryana import AsyncKuryana

# Real MDL slugs for testing (want real data)
TEST_MDLSlug = "18452-goblin"
TEST_MDLPersonSlug = "274-lee-dong-wook"
TEST_MDLUserSlug = "eggomochi"  # (if you are the user, you can request to remove)
TEST_MDLListId = "1xraljv3"


class TestAsyncKuryanaClient:
    def setup_method(self):
        self.client = AsyncKuryana()

    @pytest.mark.asyncio
    async def test_get_api(
        self,
    ):
        response = await self.client.get()
        assert response is not None
        assert "MDL Scraper API" in response.message

    @pytest.mark.asyncio
    async def test_search(self):
        query = "goblin"
        response = await self.client.search(query)
        assert response is not None
        assert len(response.results.dramas) > 0

    @pytest.mark.asyncio
    async def test_get_dram(self):
        response = await self.client.get_drama(TEST_MDLSlug)
        assert response is not None
        assert response.data.title != ""

    @pytest.mark.asyncio
    async def test_get_drama_cast(self):
        response = await self.client.get_drama_cast(TEST_MDLSlug)
        assert response is not None
        assert len(response.data.casts) > 0

    @pytest.mark.asyncio
    async def test_get_drama_episodes(self):
        response = await self.client.get_drama_episodes(TEST_MDLSlug)
        assert response is not None
        assert len(response.data.episodes) > 0

    @pytest.mark.asyncio
    async def test_get_drama_reviews(self):
        response = await self.client.get_drama_reviews(TEST_MDLSlug)
        assert response is not None
        assert len(response.data.reviews) > 0

    @pytest.mark.asyncio
    async def test_get_people(self):
        response = await self.client.get_people(TEST_MDLPersonSlug)
        assert response is not None
        assert response.data.name != ""

    @pytest.mark.asyncio
    async def test_get_user(self):
        response = await self.client.get_user(TEST_MDLUserSlug)
        assert response is not None
        assert response.data.link != ""

    @pytest.mark.asyncio
    async def test_get_list(self):
        response = await self.client.get_list(TEST_MDLListId)
        assert response is not None
        assert len(response.data.list) > 0

    @pytest.mark.asyncio
    async def test_get_seasonal_dramas(self):
        response = await self.client.get_seasonal_drama(2, 2024)
        assert response is not None
        assert len(response) > 0



from typing import List

from pydantic import BaseModel


class ListDataItem(BaseModel):
    title: str
    image: str
    rank: str
    url: str
    slug: str
    type: str
    year: str
    episodes: int
    short_summary: str


class ListData(BaseModel):
    link: str
    title: str
    description: str
    list: List[ListDataItem]


class ListQuery(BaseModel):
    slug_query: str
    data: ListData
    scrape_date: str



from typing import List, Literal
from urllib.parse import quote

import httpx

from kuryana.base import BASE_KURYANA_API_URL, BaseClient
from kuryana.types.drama import DramaQuery
from kuryana.types.drama_cast import DramaCastQuery
from kuryana.types.drama_episodes import DramaEpisodesQuery
from kuryana.types.drama_reviews import DramaReviewsQuery
from kuryana.types.index import ApiGet
from kuryana.types.list import ListQuery
from kuryana.types.people import PeopleQuery
from kuryana.types.search import SearchResultQuery
from kuryana.types.seasonal_drama import SeasonalDrama, SeasonalDramaQuery
from kuryana.types.user import UserQuery


class AsyncKuryana(BaseClient):
    def __init__(self, base_url: str | None = None) -> None:
        self._retry_count = 3
        self.client = httpx.AsyncClient(base_url=base_url or BASE_KURYANA_API_URL)

    async def _request(self, endpoint: str, **kwargs) -> httpx.Response:
        response = None
        retry_count = int(kwargs.pop("retry_count", self._retry_count))

        for _ in range(retry_count):
            response = await self.client.get(endpoint)
            if response.status_code == 200:
                return response

        if response is None:
            raise httpx.RequestError("Failed to make request after retries.")

        response.raise_for_status()
        return response

    async def get(self) -> ApiGet:
        """
        Get API
        """

        response = await self._request("/")
        return self._parse_response(response.text, class_type=ApiGet)

    async def search(self, query: str) -> SearchResultQuery:
        """
        Search for Drama / People with query
        """

        response = await self._request(f"/search/q/{quote(query)}")
        return self._parse_response(response.text, class_type=SearchResultQuery)

    async def get_drama(self, slug: str) -> DramaQuery:
        """
        Get Drama by Slug
        """

        response = await self._request(f"/id/{quote(slug)}")
        return self._parse_response(response.text, class_type=DramaQuery)

    async def get_drama_cast(self, slug: str) -> DramaCastQuery:
        """
        Get Drama Cast by Slug
        """

        response = await self._request(f"/id/{quote(slug)}/cast")
        return self._parse_response(response.text, class_type=DramaCastQuery)

    async def get_drama_episodes(self, slug: str) -> DramaEpisodesQuery:
        """
        Get Drama Episodes by Slug
        """

        response = await self._request(f"/id/{quote(slug)}/episodes")
        return self._parse_response(response.text, class_type=DramaEpisodesQuery)

    async def get_drama_reviews(self, slug: str, page: int = 1) -> DramaReviewsQuery:
        """
        Get Drama Reviews by Slug
        """

        if page < 1:
            page = 1

        response = await self._request(f"/id/{quote(slug)}/reviews?page={page}")
        return self._parse_response(response.text, class_type=DramaReviewsQuery)

    async def get_people(self, slug_id: str) -> PeopleQuery:
        """
        Get People by ID
        """

        response = await self._request(f"/people/{quote(slug_id)}")
        return self._parse_response(response.text, class_type=PeopleQuery)

    async def get_user(self, user_id: str) -> UserQuery:
        """
        Get User by ID
        """

        response = await self._request(f"/dramalist/{quote(user_id)}")
        return self._parse_response(response.text, class_type=UserQuery)

    async def get_list(self, list_id: str) -> ListQuery:
        """
        Get Public Lists by ID
        """

        response = await self._request(f"/list/{quote(list_id)}")
        return self._parse_response(response.text, class_type=ListQuery)

    async def get_seasonal_drama(
        self, season: Literal[1, 2, 3, 4], year: int
    ) -> List[SeasonalDrama]:
        """
        Get Seasonal Drama by Year and Season

        Season: 1 = Winter, 2 = Spring, 3 = Summer, 4 = Fall
        """

        response = await self._request(f"/seasonal/{year}/{season}")
        return self._parse_array_response(
            response.text, adapter_type=SeasonalDramaQuery
        )



from datetime import datetime
from typing import Optional

from pydantic import BaseModel


class DramaReviewsReviewer(BaseModel):
    name: str
    user_link: str
    user_image: str
    info: str


class DramaReviewsReview(BaseModel):
    reviewer: DramaReviewsReviewer
    review: Optional[list[str]] = None
    ratings: Optional[dict[str, float]] = None


class DramaReviewsData(BaseModel):
    link: str
    title: str
    poster: str
    reviews: list[DramaReviewsReview]


class DramaReviewsQuery(BaseModel):
    slug_query: str
    data: DramaReviewsData
    scrape_date: datetime



from datetime import datetime
from typing import Dict, List, Optional

from pydantic import BaseModel


class DramaCastRoleClass(BaseModel):
    name: str
    type: str


class DramaCastRole(BaseModel):
    name: str
    profile_image: str
    slug: str
    link: str
    role: Optional[DramaCastRoleClass] = None


class DramaCastData(BaseModel):
    link: str
    title: str
    poster: str
    casts: Dict[str, List[DramaCastRole]]


class DramaCastQuery(BaseModel):
    slug_query: str
    data: DramaCastData
    scrape_date: datetime



from datetime import datetime
from typing import Optional

from pydantic import BaseModel


class DramaEpisodesEpisode(BaseModel):
    title: str
    image: str
    link: str
    rating: Optional[str] = None
    air_date: Optional[str] = None


class DramaEpisodesData(BaseModel):
    title: str
    episodes: list[DramaEpisodesEpisode]


class DramaEpisodesQuery(BaseModel):
    slug_query: str
    data: DramaEpisodesData
    scrape_date: datetime



from datetime import datetime
from typing import List, Optional, Union

from pydantic import BaseModel


class SearchDrama(BaseModel):
    slug: str
    thumb: str
    mdl_id: str
    title: str
    ranking: Optional[str] = None
    type: str
    year: Optional[int] = None
    series: Union[bool, str]


class SearchPerson(BaseModel):
    slug: str
    thumb: str
    name: str
    nationality: str


class SearchResults(BaseModel):
    dramas: List[SearchDrama]
    people: List[SearchPerson]


class SearchResultQuery(BaseModel):
    query: str
    results: SearchResults
    scrape_date: datetime



from datetime import datetime
from typing import List, Optional

from pydantic import BaseModel


class DramaCast(BaseModel):
    name: str
    profile_image: str
    slug: str
    link: str


class DramaDetails(BaseModel):
    country: str
    type: str
    episodes: str
    aired: str
    original_network: str
    duration: str
    score: str
    ranked: str
    popularity: str
    content_rating: str
    watchers: str
    favorites: str


class DramaOthers(BaseModel):
    related_content: List[str]
    native_title: List[str]
    also_known_as: List[str]
    genres: List[str]
    tags: List[str]


class DramaData(BaseModel):
    link: str
    title: str
    complete_title: str
    sub_title: str
    year: Optional[str] = None
    rating: float
    poster: str
    synopsis: str
    casts: List[DramaCast]
    details: DramaDetails
    others: DramaOthers


class DramaQuery(BaseModel):
    slug_query: str
    data: DramaData
    scrape_date: datetime
